package org.apidesign.apifest08.currency;

/**
 * ConvetorImpl it the basic implementaion of Convertor interface.
 * @see Convertor Convertor for more details.
 * The 100 pences makes 1 amount of the currency.
 *
 * @author jindra
 */
final class ConvertorImpl implements Convertor {

    private static final double P_TO_AM = 100;
    private double exchangeRate;

    ConvertorImpl(double exchangeRate) {
        this.exchangeRate = exchangeRate;
    }

    public Amount convert(Amount amount) {
        verifyInput(amount);
        double result = convertToDouble(amount) * exchangeRate;
        return convertToAmount(result);
    }

    public Amount convertBack(Amount amount) {
        if (amount == null) {
            throw new IllegalArgumentException("Amount must be not null");
        }
        double result = convertToDouble(amount) / exchangeRate;
        return convertToAmount(result);
    }

    private double convertToDouble(Amount amount) {
        double am = amount.getAmount();
        double pc = amount.getPence();
        return am + (pc / P_TO_AM);
    }

    private Amount convertToAmount(double result) {
        long resultAm = Math.round(Math.floor(result));
        long resultPc = Math.round(Math.floor((result * P_TO_AM - resultAm * P_TO_AM)));
        return new Amount(resultAm, resultPc);
    }

    private void verifyInput(Amount amount) {
        if (amount == null) {
            throw new IllegalArgumentException("Amount must be not null");
        }
        if (amount.getPence() < 0) {
            throw new IllegalArgumentException("Pences must not be negative");
        }
        if (amount.getPence() > P_TO_AM) {
            throw new IllegalArgumentException("Pences must not be over P_TO_AM");
        }
    }
}
